REIN version 1.0 Release Notes
======================================================================

Contents

  - Introduction
     1. Environments
     2. Spawn and non spawn options
     3. MD Programs
     4. Tools
  - Compiling REIN
     1. Main code
     2. Tools (Supporting Softwares)
  - Running REIN
     1. Input
     2. Batch script for job scheduler
  - Manuplation of the trajectory files

----------------------------------------------------------------------
Release:
v.1.0     10/31/2012
     .01  11/31/2012 for flat mpi namd2, small bug fix
     .02  02/04/2013 for gfortran, small bug fix

Introduction
----------------------------------------------------------------------

__REIN: Replica-exchange Interface Program__

  REIN contains REIN main program, Supporting programs (tools), and 
  tutorials. Three supporting softwares and one scripts are contained 
  in tool/ directory. See Tools.

  REIN supports Replica-exchange molecular dynamics (REMD), 
  Replica-exchange umbrella sampling (REUS), and Multi-dimensional 
  Replica-exchange molecular dynamics simulations (MREM). REUS 
  supports the distance, angle and dihedral angle constraints. The 
  discription of the selection atoms depends on MD program.

### 1. Environments ##################################################

 - __Machines:__  
     - PC cluster, K computer, FX10
 - __Scheduler (batch_script):__ 
     - Grid Engine (GE), fujitsu job scheduler (K, FX10)
 - __Program Language:__ 
     - Fortran 95/2003 (intel fortran, fujitsu fortran, gfortran), 
     - MPI-2 (Open MPI, Fujitsu MPI)
 - __Parallelization:__ 
     - MPI / OpenMP hybrid

### 2. Spawn and Non spawn options ###################################

  REIN has two options, spawn and non spawn, for compilation. 
  Default is "Spawn" option.
  See compilation in [ Compiling REIN ] and [ Running REIN ]

#### - Spawn option

  It is for the huge PC clusters, K computer, FX10 and so on. The 
  system needs shared HDD, likes NFS. It uses the function of MPI-2 
  (mpi_comm_spawn). So, it is depend on the scheduler and cluster 
  systems. Please ask system administrator. K computer, FX10 are 
  supported. 
  
  MD program should be compiled by MPI. 
  Please download MD source code and compile it by MPI.

### 3. MD programs ###################################################

  REIN is the interface program and needs the conventional MD program 
  packages. REIN supports NAMD and MARBLE currently. Please download 
  MD packages and compile it. You can download the binary package of 
  NAMD: __Linux-x86_64-multicore__ , 
  if you use non spawn option.

|  options  |   MD   |  MPI  | Multi-node MD | Compilation   |
|:---------:|:-------|:-----:|:-------------:|:-------------:|
|  spawn    |  NAMD  |  Yes  |      Yes      | Necessary     |
| non spawn |  NAMD  |  No   |  No (1 node)  | Not necessary |

#### Download programs from their websites

  - __NAMD:__
[Download](http://www.ks.uiuc.edu/Development/Download/download.cgi?PackageName=NAMD)
  - __MARBLE:__
[Download] (http://www.tsu    rumi.yokohama-cu.ac.jp/bioinfo/marble/)

  __Note) NAMD on K, FX10: Please contact us (<miya@riken.jp>), if__
  __you would like to use NAMD binary program on K or FX10.__

### 4. Tools (Supporting softwares) ##################################

  We have prepared three supporting softwares, input_builder, 
  batch_builder, and remd_converter, and one scripts for the chain job
  for jobscheduler, auto_submitter.pl. These softwares supports the 
  performing REIN. 

  - __input_builder:__   
    It creates the input files and directories for REIN automatically. 
    Input files of REIN is very complicated, to support several 
    functions. To create input files easily, you should use it.

  - __batch_builder:__  
    It crates the batch scripts for job scheduler. Grid Engine (GE), K 
    computer/FX10 scheduler and Fujitsu scheduler are supported. You 
    can easily add the function to create the batch scripts for other 
    job scheduler, adding template file.

  - __remd_converter:__  
    This software is for creating sequential trajectories. 

----------------------------------------------------------------------


Compiling REIN
----------------------------------------------------------------------

### 1. Main code #####################################################

#### - configure

    $ cd src

  For x86_64 machine with intel fortran 

    $ ./configure.sh x86_64 ifort

  For x86_64 machine with gfortran 

    $ ./configure.sh x86_64 gfortran

  For sparc machine with fujitsu compilar 

    $ ./configure.sh sparc fujitsu

  For K computer 

    $ ./configure.sh sparc k

  Compilation for non-spawn version
    
    $ ./configure.sh x86_64 ifort local

#### - make/install

    $ make; make install

  Then, bin/ directory is created, and rein binary file is copied into 
  bin/ directory.

#### - uninstall

    $ make distclean


### 2. Tools #########################################################

#### - configure

    $ cd ../tools

  For x86_64 machine with intel fortran 
    $ ./configure.sh x86_64 ifort

  For x86_64 machine with gfortran 
    $ ./configure.sh x86_64 gfortran

#### - make/install

    $ make; make install

  Then, bin/ directory is created, and input builder, batch builder, 
  remd converter, and templates are copied into bin/ directory.

#### - uninstall

    $ make distclean


Running REIN
----------------------------------------------------------------------

  Here we show the example. Please see detail in example.

### 1. Prepareing initial files. #####################################

  Please prepare the initial files for your system. Restart files 
  and structure and topology files are necessary.
  If you use input_builder and batch_builder, all initial file name
  should be "initial"."(extention)", and You need to copy all files and 
  directories in rein/bin to the directory for the production.
  Ex)

    $ cp -r rein/bin/* .

  Note) rein/bin/templates directory is also necessary to use tools.

#### - Input Files of REIN

  Input files for REIN and input files for MD are nesessary.
  if you perform REIN calculation on "sample" directory, input files
  and directories are following:
  ( Example MD: NAMD, 4 replicas )

    sample/ ------ rein.inp                   (For REIN)
              |
              |--- file_rank.d                (For REIN)
              |--- file_value.d               (For REIN)
              |--- file_on.d                  (For REIN)
              |--- file_axis_ex.d             (For REIN)
              |--- file_coef.d                (For REIN)
              |
              |--- initial.pdb                (NAMD)
              |--- initial.psf                (NAMD)
              |--- par_all27_prot_lipid.prm   (NAMD)
              |--- initial.inp                (NAMD modified)
              |--- initial.colvar  [for REUS] (NAMD modified)
              |
              |--- rep.1/ ----- rep.1.0.coor  (NAMD)   
              |            |--- rep.1.0.vel   (NAMD)
              |            `--- rep.1.0.xsc   (NAMD)
              |--- rep.2/ ----- rep.2.0.coor  (NAMD)
              |            |--- rep.2.0.vel   (NAMD)
              |            `--- rep.2.0.xsc   (NAMD)
              |--- rep.3/ ----- rep.3.0.coor  (NAMD)
              |            |--- rep.3.0.vel   (NAMD)
              |            `--- rep.3.0.xsc   (NAMD)
              `--- rep.4/ ----- rep.4.0.coor  (NAMD)
                           |--- rep.4.0.vel   (NAMD)
                           `--- rep.4.0.xsc   (NAMD)


| Default filename |  Description                                    |
|:-----------------|:------------------------------------------------|
| rein.inp         | Inputs to perform REIN                          |
|                  |                                                 |
| file_rank.d      | List of initial rank                            |
| file_value.d     | List of consition parameter                     |
| file_on.d        | List of active rank                             |
| file_ex.d        | List of exchange pattern                        |
| file_axis_ex.d   | List of exchange order for axis                 |
| file_coef.d      | List of rescaling parameter for temperature     |
|                  |                                                 |
| initial.pdb      | pdb (structure files)                           |
| initial.psf      | psf files (topology files) for NAMD             |
| par_all27*.inp   | Parameter files for Force field                 |
| initial.inp      | Template file for namd input file with rein tag |
| initial.colver   | Template file for namd colver file with rein tag|
|                  |                                                 |
| rep.# directory  | The input/output files of replica #             |
| rep.#.0.coor     | The restart file for the coordinate             |
| rep.#.0.vel      | The restart file for the velocity               |
| rep.#.0.xsc      | The restart file for the box information        |

  The detail infomation of files is described in user manual.
  Input files for MD should be created by MD simulations.

#### - Input builder

  Input builder creates avobe imput files and directories by very 
  easy inputs.

  1. Copy all files and directory in rein/bin to your current
  directory.

  2. Copy initial.pdb, initial.psf, initial.coor, initial.vel,
  initial.xsc and par_all27_prot_lipid.inp to your current directory.

  3. Then, create the template input file by input_builder.

        $ ./input_builder -h ctrl > ib.inp

  4. Modify the ib.inp

  5. Run input_builder
  
        $ ./input_builder ib.inp

  Then you can obtain necessary input files for rein by input_builder.


### 2. Batch script for job scheduler (K/FX10/GE)#####################

  Batch_builder can easily creates batch scripts for job scheduler. It
  supports job schedulers for K and FX10 and Grid Engine (Sun grid 
  engine etc..). If you modify the templates files, you can easily 
  extend it for other jobschedulers.

#### - Batch builder

  1. Copy all files and directory in rein/bin to your current
     directory.(You can skip it, if you already copied it before.)

  2. Create the template input file by batch_builder.

        $ ./batch_builder -h ctrl > bb.inp

  4. Modify the bb.inp

  5. Run batch_builder
  
        $ ./batch_builder bb.inp

  Then you can obtain batch scripts for job_scheduler.

#### - Job submittion

  Submitting the rein in GE

      $ qsub rein.sh

  Submitting the rein in K and FX10

      $ pjsub rein.sh

#### - Job restarting

  It can easily creates restart input files.

      $ ./batch_builder bb.inp restart

  All Output files are copied to results/ directory.
  Then you can submit rein again.

      $ pjsub rein.sh



Manuplation of the trajectory files
----------------------------------------------------------------------

  REIN produces a lot of snapshots of the structure in the mrem 
  calculations. remd_converter connects the snapshots and creates the 
  trajectories for conditions or replicas.

    $ ./remd_converter -h ctrl > rc.inp

  Modify the rc.inp. and run remd_converter.

    $ ./remd_converter rc.inp


----------------------------------------------------------------------

