!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module : constants_mod
!> @brief   collection of REIN constant parameters
!! @author  Naoyuki Miyashita (NM)
!! @date    NM: 2009/04/21
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module constants_mod

  implicit none

  public

  !! dim of sending data array for exchange
  integer,            parameter :: idata      =        7  

  integer,            parameter :: io_num_1   =    90000
  integer,            parameter :: io_num_2   =    70000
  integer,            parameter :: imega      =  1000000
  integer,            parameter :: ihkiro     =   100000
  integer,            parameter :: imax       =   100000
  integer,            parameter :: imax_ten   =       10
  integer,            parameter :: imax_1h    =      100
  integer,            parameter :: imax_1k    =     1000
  integer,            parameter :: imax_10k   =    10000

  integer,            parameter :: imax_replicas = 10000
  integer,            parameter :: imax_axis     =    10
  integer,            parameter :: imax_param    =     5

  integer,            parameter :: icha_8     =   8
  integer,            parameter :: icha_16    =  16
  integer,            parameter :: icha_line  =  80
  integer,            parameter :: icha_ten   =  10
  integer,            parameter :: icha_1h    = 100
  integer,            parameter :: icha_2h    = 200
  integer,            parameter :: icha_3h    = 300

  integer,            parameter :: io_in      =  5
  integer,            parameter :: io_out     =  6

  integer,            parameter :: tmp_io1    = 11
  integer,            parameter :: tmp_io2    = 12
  integer,            parameter :: tmp_io3    = 13
  integer,            parameter :: tmp_io4    = 14
  integer,            parameter :: tmp_io5    = 15
  integer,            parameter :: tmp_io6    = 16
  integer,            parameter :: tmp_io7    = 17
  integer,            parameter :: tmp_io8    = 18
  integer,            parameter :: tmp_io9    = 19
  integer,            parameter :: tmp_io10   = 20
  integer,            parameter :: tmp_io11   = 21
  integer,            parameter :: tmp_io12   = 22
  integer,            parameter :: tmp_io13   = 23
  integer,            parameter :: tmp_io14   = 24
  integer,            parameter :: tmp_io15   = 25
  integer,            parameter :: tmp_io16   = 26
  integer,            parameter :: tmp_io17   = 27
  integer,            parameter :: tmp_io18   = 28
  integer,            parameter :: tmp_io19   = 29
  integer,            parameter :: tmp_io20   = 30
  integer,            parameter :: tmp_io21   = 31

  double precision,   parameter :: err16      = 1.0D-16
  double precision,   parameter :: err10      = 1.0D-10

  double precision,   parameter :: kboltz     = 1.987191D-03
  double precision,   parameter :: jkboltz    = 1.380662D-23
  double precision,   parameter :: avogadro   = 6.022045D23
  double precision,   parameter :: jkcal      = 4186.05D0
  double precision,   parameter :: kcalmol    = jkcal/avogadro
  double precision,   parameter :: pi         = 3.14159265358979323846D0

  integer,            parameter ::  gl_max_cycle = 3000

  character(icha_2h), parameter :: list_program(1:5) =  &
       (/ "mpi-namd"      &
         ,"chm-namd"      &
         ,"cudanamd"      &
         ,"mpicharm"      &
         ,"sglcharm"      &
       /)

  character(icha_2h), parameter :: axis_type_list(1:5) = &
       (/ &
          "temperature"       &
         ,"harmonic   "       &
         ,"pressure   "       &
         ,"volume     "       &
         ,"quad       "       &
       /)

end module constants_mod
