!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------0---------0---------0---------0---------0---------0---------0---------0
!
!  Program: batch_builder
!
!  Description: build REIN job script
!
!  Authors: Norio Takase (NT), Naoyuki Miyashita (NM)
!
!  Date: 29 May, 2012
!
!--------0---------0---------0---------0---------0---------0---------0---------0

program batch_builder

  use bb_batch_mod
  use bb_reininp_modify_mod
  use bb_restart_mod
  use bb_reininp_mod
  use bb_ctrl_mod
  use bb_usage_mod

  implicit none

  ! local variables
  character(100)    :: ctrl_filename
  type(s_ctrl_data) :: ctrl_data
  type(s_rein_inp)  :: rein_inp
  logical           :: rst_flag
  integer           :: rst_step


  ! show usage
  !
  call usage(ctrl_filename, rst_flag, rst_step)


  ! read control parameters for batch builder
  !
  write(MsgOut,'(A)') '[STEP1] Read Control Parameters for batch builder'
  write(MsgOut,'(A)') ' '

  call control(ctrl_filename, ctrl_data)


  ! process for REIN restart
  !
  write(MsgOut,'(A)') '[STEP2] Process for REIN restart'
  write(MsgOut,'(A)') ' '

  if (rst_flag .eqv. .true.) then

    call reininp(ctrl_data, rein_inp)
    call restart(ctrl_data, rein_inp, rst_step)

  else
    write(MsgOut,'(A)') ' ... skip. '
    write(MsgOut,'(A)') ' '

  end if


  ! parse REIN input file
  !
  write(MsgOut,'(A)') '[STEP3] Parse REIN input file'
  write(MsgOut,'(A)') ' '

  if (rst_flag .eqv. .false.) then

    call reininp(ctrl_data, rein_inp)
    call reininp_modify(ctrl_data, rein_inp)

  end if

  call reininp(ctrl_data, rein_inp)


  ! build batch script file
  !
  write(MsgOut,'(A)') '[STEP4] build batch script file'
  write(MsgOut,'(A)') ' '

  call batch(ctrl_data, rein_inp)

      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*    BATCH_BUILDER: Bye                            *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '

  stop

end program batch_builder
