!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   bb_usage_mod
!> @brief   show usage of batch builder
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM)
!! @date    2012/05/30
! 
!--------1---------2---------3---------4---------5---------6---------7---------8

module bb_usage_mod

  use messages_mod

  implicit none

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  usage
  !> @brief      show usage of batch_builder
  !! @authors    NM, NT
  !! @param[out] ctrl_filename  control file name
  !! @param[out] rst_flag       flag for restart or not
  !! @param[out] rst_step       step number for restart
  !! @date       2012/05/30 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine usage(ctrl_filename, rst_flag, rst_step)

    ! formal arguments
    character(*), intent(out) :: ctrl_filename
    logical,      intent(out) :: rst_flag
    integer,      intent(out) :: rst_step

    ! local variables
    character(80)             :: arg1, arg2, arg3
    

    call getarg(1, arg1)
    call getarg(2, arg2)
    call getarg(3, arg3)

    if (iargc() < 1      .or. &
         arg1 == '-help' .or. &
         arg1 == '-h'    .or. &
         arg1 == '-HELP' .or. &
         arg1 == '-H') then

      if (iargc() < 2 .or. arg2 /= 'ctrl') then
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to first run'
        write(MsgOut,'(A)') '  % ./batch_builder INPBB'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to restart'
        write(MsgOut,'(A)') '  % ./batch_builder INPBB restart [step number]'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to see control parameters'
        write(MsgOut,'(A)') '  % ./batch_builder -h ctrl'
        write(MsgOut,'(A)') ' '
      else
        if (arg3 /= 'all') then
          write(MsgOut,'(A)') '# control parameters in batch_builder'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'rein_input             = rein.inp       '//&
               '# REIN input file name'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '# chain_job            = yes            '//&
               '  # (YES/[NO]) It is for chain_job. For k:(NO)'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') '[BATCH]'
          write(MsgOut,'(A)') 'batchfile_name         = rein.sh        '//&
               '# batch filename '
          write(MsgOut,'(A)') 'job_scheduler          = K              '//&
               '# job scheduler type ([K]/GE/RICC)'
          write(MsgOut,'(A)') 'queue_name             = small          '//&
               '# queue name, k:([SMALL]/LARGE),fx10: REGULAR..'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') 'num_threads_md         = 1              '//&
               '# number of threads in md'
          write(MsgOut,'(A)') 'num_mpiproc_md         = 8              '//&
               '# # of mpi process for md'
          write(MsgOut,'(A)') 'cpu_time               = 01:00:00       '//&
               '# cpu time'
          write(MsgOut,'(A)') 'num_core_cpu           = 8              '//&
               '# number of core in cpu, for k = 8, for fx10 = 16, ... '
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') '[REMD]'
          write(MsgOut,'(A)') 'replica_exchange       = yes            '//&
               '# ([YES]/NO): replica exchange / umbrella sampling'
          write(MsgOut,'(A)') 'num_of_exchange_steps  = 1              '//&
               '# number of exchange steps'
          write(MsgOut,'(A)')' '
        else
          write(MsgOut,'(A)') '# control parameters in batch_builder'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'rein_input             = rein.inp       '//&
               '# REIN input file name'
          write(MsgOut,'(A)') 'skip_modify_input      = no             '//&
               '# skip the modifying REIN input file'
          write(MsgOut,'(A)') 'directory              = .              '//&
               '# Working directory for REIN'
          write(MsgOut,'(A)') 'template_dir           =                '//&
               '# empty => <program_path>/templates'
          write(MsgOut,'(A)') 'parallel               = mpiexec        '//&
               '# parallel program for REIN'
          write(MsgOut,'(A)') 'batch_builder_exe      = ./batch_builder'//&
               ' # batch_builder programm name'
          write(MsgOut,'(A)') 'batch_builder_input    = INPBB          '//&
               '# input file name for batch_builder'
          write(MsgOut,'(A)') 'spawn                  = yes            '//&
               '# ([YES]/NO) uses spawn or not'
          write(MsgOut,'(A)') 'openmp_rein            = no             '//&
               '# (YES/[NO]) uses OpenMP or not'
          write(MsgOut,'(A)') 'gpgpu                  = no             '//&
               '# (YES/[NO]) uses GPGPU or not'
          write(MsgOut,'(A)') 'hybrid_md              = no             '//&
               '# (YES/[NO]) uses hybrid_md or not'
          write(MsgOut,'(A)') '# chain_job              = yes          '//&
               '  # (YES/[NO]) It is for chain_job. For k:(NO)'
          write(MsgOut,'(A)') '                                        '
          write(MsgOut,'(A)') '[BATCH]                                 '
          write(MsgOut,'(A)') 'batchfile_name         = rein.sh        '//&
               '# batch filename '
          write(MsgOut,'(A)') 'job_scheduler          = K              '//&
               '# job scheduler type ([K]/GE/RICC)'
          write(MsgOut,'(A)') 'queue_name             = small          '//&
               '# queue name, k:([SMALL]/LARGE),fx10: REGULAR..'
          write(MsgOut,'(A)') 'num_threads_md         = 1              '//&
               '# number of threads in md'
          write(MsgOut,'(A)') 'num_mpiproc_md         = 8              '//&
               '# # of mpi process for md'
          write(MsgOut,'(A)') 'cpu_time               = 01:00:00       '//&
               '# cpu time'
          write(MsgOut,'(A)') 'num_core_cpu           = 8              '//&
               '# number of core in cpu, for k = 8, for fx10 = 16, ... '
          write(MsgOut,'(A)') 'num_mpiproc_rein       = 1              '//&
               '# # of mpi process for REIN'
          write(MsgOut,'(A)') 'stage_out_dir          = ./STAGE_OUT    '//&
               '# directory for staging-out'
          write(MsgOut,'(A)') 'machine_file           =                '//&
               '# machine file for REIN parallel exec.'
          write(MsgOut,'(A)') 'direct_describe        = no             '//&
               '# (YES/[NO]) node/process direct describe for K/UGE'
          write(MsgOut,'(A)') 'node                   = 9              '
          write(MsgOut,'(A)') 'process                = 9              '
          write(MsgOut,'(A)') 'mpi_static_node_shape  = 1              '
          write(MsgOut,'(A)') 'mpi_static_node_proc   = 8              '
          write(MsgOut,'(A)') '                                        '
          write(MsgOut,'(A)') '[REMD]                                  '
          write(MsgOut,'(A)') 'replica_exchange       = yes            '//&
               '# ([YES]/NO): replica exchange / umbrella sampling'
          write(MsgOut,'(A)') 'num_of_exchange_steps  = 1              '//&
               '# number of exchange steps'
          write(MsgOut,'(A)') 'first_step             = 1              '//&
               '# first step number'
          write(MsgOut,'(A)') 'num_of_product_steps   = 1              '//&
               '# # of product steps'
          write(MsgOut,'(A)') 'num_of_equiv_steps     = 1              '//&
               '# # of equiv steps'
          write(MsgOut,'(A)')' '
          
        end if
      endif

      stop

    else
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*    BATCH_BUILDER: build batch script for REIN    *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '

      ctrl_filename = arg1

      rst_flag = arg2 == 'restart' .or. &
                 arg2 == 'RESTART' .or. &
                 arg2 == 'Restart'

      rst_step = 0
      if (arg3 /= '') then
        read(arg3,*,err=100,end=100) rst_step
        return
100     write(MsgOut,'(A,A,A)') 'Usage> Warning: Bad restart step number: "',&
             TRIM(arg3), '"'
        write(MsgOut,'(A)')   ' '
      end if

    end if
    
    return
  
  end subroutine usage

end module bb_usage_mod
