!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   ib_box2pme
!> @brief   factorization for the translation from box size to pme mesh
!! @authors Naoyuki Miyashita (NM)
!! @date    2012/10/02
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module ib_box2pme

  implicit none
  private
  
  public::  box2pme

contains


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  division
  !> @brief      value is devided by ifact
  !! @authors    NM
  !! @date       2012/10/02 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine ifactor (value, ifact, istock, imod)

    integer,          intent(in)   :: value
    integer,          intent(in)   :: ifact
    integer,          intent(out)  :: istock
    integer,          intent(out)  :: imod

    integer                        :: i

    istock = value

    do i = 1, 100,1

      imod = mod ( istock, ifact )

      if (imod == 0) then

        istock = int ( istock / ifact ) 

      else

        exit

      end if

    end do
    
    return
  end subroutine ifactor


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  factor_235
  !> @brief      value is devided by 2, 3, 5
  !! @authors    NM
  !! @date       2012/10/02 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine factor_235 (value, istock, imod)
   
    integer,          intent(in)   :: value
    integer,          intent(out)  :: istock
    integer,          intent(out)  :: imod

    integer                        :: ifact

    istock = value 
    ! 2
    ifact = 2
    call ifactor (istock ,ifact, istock, imod)

    ! 3
    ifact = 3
    call ifactor (istock ,ifact, istock, imod)
    
    ! 5
    ifact = 5
    call ifactor (istock ,ifact, istock, imod)
   
    return 
  end subroutine factor_235


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  search_rearest
  !> @brief      searching the rearest pme value
  !! @authors    NM
  !! @date       2012/10/02 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine search_nearest ( value, istock, imod, new_value )
   
    integer,          intent(in)   :: value
    integer,          intent(out)  :: istock
    integer,          intent(out)  :: imod
    integer,          intent(out)  :: new_value

    integer                        :: ifact
    integer                        :: new_factor
    integer                        :: i

    new_factor = value
 
    do i = 1, 10000, 1

      call factor_235 ( new_factor, istock, imod )

      if ( istock == 1 .and. imod == 1 ) then

        exit

      else

        new_factor = new_factor + 1

      end if
      
    end do

    new_value =  new_factor
 
    return
  end subroutine search_nearest


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  box2pme
  !> @brief      translation from box size (real) to pme_mesh (integer)
  !! @authors    NM
  !! @date       2012/10/02 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine box2pme ( box_size, pme_mesh )

    real*8,             intent(in)     :: box_size
    integer,            intent(out)    :: pme_mesh

    integer                            :: ibox_size
    integer                            :: ivalue, imod

    ibox_size = int ( box_size + 0.999d0 )

    call search_nearest ( ibox_size, ivalue, imod, pme_mesh )

    return
  end subroutine box2pme 

end module ib_box2pme

