!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   ib_build_mod
!> @brief   build REIN input files
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM)
!! @date    2012/06/11
! 
!--------1---------2---------3---------4---------5---------6---------7---------8

module ib_build_mod

  use ib_build_mdinp_mod
  use ib_build_reininp_mod
  use ib_build_file_d_mod
  use ib_copy_file_mod
  use ib_setup_mod
  use ib_ctrl_mod
  use replace_string_mod
  use fileio_control_mod
  use fileio_mod
  use messages_mod

  implicit none
  private
  
  public  :: build
  private :: check_files

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  build
  !> @brief      build REIN input files
  !! @authors    NT
  !! @param[in]  ctrl_data  control parameters in the program
  !! @param[in]  build_info build information
  !! @date       2012/06/11 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine build(ctrl_data, build_info)

    ! formal arguments
    type(s_ctrl_data),  intent(in) :: ctrl_data
    type(s_build_info), intent(in) :: build_info

   
    ! check files
    !
    call check_files(ctrl_data)


    ! build file.*.d
    !
    call build_file_d(ctrl_data, build_info)


    ! build REIN input file
    !
    call build_reininp(ctrl_data, build_info)


    ! build MD input files
    !
    call build_mdinp(ctrl_data, build_info)


    ! copy files to replica directories
    !
    call copy_files(ctrl_data, build_info)

    return
  
  end subroutine build

end module ib_build_mod
