!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   ib_usage_mod
!> @brief   show usage of batch builder
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM)
!! @date    2012/06/06
! 
!--------1---------2---------3---------4---------5---------6---------7---------8

module ib_usage_mod

  use messages_mod

  implicit none

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  usage
  !> @brief      show usage of input_builder
  !! @authors    NM,NT
  !! @param[out] ctrl_filename  control file name
  !! @date       2012/06/14 
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine usage(ctrl_filename)

    ! formal arguments
    character(*), intent(out) :: ctrl_filename

    ! local variables
    character(80)             :: arg1, arg2, arg3
    

    call getarg(1, arg1)
    call getarg(2, arg2)
    call getarg(3, arg3)

    if (iargc() < 1      .or. &
         arg1 == '-help' .or. &
         arg1 == '-h'    .or. &
         arg1 == '-HELP' .or. &
         arg1 == '-H') then

      if (iargc() < 2 .or. arg2 /= 'ctrl') then
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to first run'
        write(MsgOut,'(A)') '  % ./input_builder INPIB'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to see control parameters'
        write(MsgOut,'(A)') '  % ./input_builder -h ctrl'
        write(MsgOut,'(A)') ' '
      else
        if (arg3 /= 'all') then
          write(MsgOut,'(A)') '# control parameters in input_builder'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'rein_input            = rein.inp      '//&
               '# rein input file name'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '[REMD]'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'axis001               = temperature   '//&
               '# axis (TEMPERATURE/HARMONIC)'
          write(MsgOut,'(A)') 'num_replicas001       = 8             '//&
               '# # of replicas for the axis'
          write(MsgOut,'(A)') 'range001              = 300.0 304.0   '//&
               '# lowest, highest value'
          write(MsgOut,'(A)') 'division001           = exp           '//&
               '# (EQUIV/EXP/CYCLE)'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '# axis002               = harmonic'
          write(MsgOut,'(A)') '# num_replicas002       = 8'
          write(MsgOut,'(A)') '# range002              = 2.0 8.0'
          write(MsgOut,'(A)') '# division002           = equiv'
          write(MsgOut,'(A)') '# distance002           = ADP CR 1-1 ADP CL 1-1'
          write(MsgOut,'(A)') '# angle002              = '
          write(MsgOut,'(A)') '# dihedral002           = ADP NL 1-1  '//&
               'ADP CA 1-1  ADP CRP 1-1  ADP NR 1-1'
          write(MsgOut,'(A)') '# spring_const002       = 2.0'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '[MD]'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') 'md_program            = namd          '//&
               '# md program (NAMD/MARBLE)'
          write(MsgOut,'(A)') 'md_exe_file           = ./namd2       '//&
               '# md exe file'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') 'md_temperature        = 300.0         '//&
               '# md temperature'
          write(MsgOut,'(A)') 'md_steps              = 1000          '//&
               '# md steps / exchange'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') 'system_size_x         = 23.4          '//&
               '# system size --automatically create pme grids'
          write(MsgOut,'(A)') 'system_size_y         = 23.2          '//&
               '#'
          write(MsgOut,'(A)') 'system_size_z         = 23.9          '//&
               '#'
          write(MsgOut,'(A)')' '
          write(MsgOut,'(A)') '# pme_grid              = yes           '//&
               '# if you want to setup pme grids, (YES/[NO]) '
          write(MsgOut,'(A)') '# pme_grid_size_x       = 24            '//&
               '# pme grid size x, y, z'
          write(MsgOut,'(A)') '# pme_grid_size_y       = 24            '//&
               '#'
          write(MsgOut,'(A)') '# pme_grid_size_z       = 24            '//&
               '#'
          write(MsgOut,'(A)')' '
        else
          write(MsgOut,'(A)') '# control parameters in input_builder'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'rein_input            = rein.inp      '//&
               '# rein input file name'
          write(MsgOut,'(A)') 'directory             = .             '//&
               '# working directory for REIN'
          write(MsgOut,'(A)') 'template_dir          =               '//&
               '# empty => <program_path>/templates'
          write(MsgOut,'(A)') 'spawn                 = yes           '//&
               '# ([YES]/NO) uses spawn or not'
          write(MsgOut,'(A)') 'parallel_module_md    =               '//&
               '# parallel module for MD'
          write(MsgOut,'(A)') 'option_md             =               '//&
               '# option for MD'
          write(MsgOut,'(A)') 'extend_option_md      =               '//&
               '# extend option for MD'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '[REMD]'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'axis001               = temperature   '//&
               '# axis (TEMPERATURE/HARMONIC)'
          write(MsgOut,'(A)') 'num_replicas001       = 8             '//&
               '# # of replicas for the axis'
          write(MsgOut,'(A)') 'range001              = 300.0 304.0   '//&
               '# lowest, highest value'
          write(MsgOut,'(A)') 'division001           = exp           '//&
               '# (EQUIV/EXP/CYCLE)'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '# axis002               = harmonic'
          write(MsgOut,'(A)') '# num_replicas002       = 8'
          write(MsgOut,'(A)') '# range002              = 2.0 8.0'
          write(MsgOut,'(A)') '# division002           = equiv'
          write(MsgOut,'(A)') '# distance002           = ADP CR 1-1 A'//&
               'DP CL 1-1'
          write(MsgOut,'(A)') '# angle002              = '
          write(MsgOut,'(A)') '# dihedral002           = ADP NL 1-1  '//&
               'ADP CA 1-1  ADP CRP 1-1  ADP NR 1-1'
          write(MsgOut,'(A)') '# spring_const002       = 2.0'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'first_step            = 1             '//&
               '# first step number'
          write(MsgOut,'(A)') 'product_steps         = 0             '//&
               '# # of product steps'
          write(MsgOut,'(A)') 'equiv_steps           = 0             '//&
               '# # of equiv steps'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') '[MD]'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'md_program            = namd          '//&
               '# md program (NAMD/MARBLE)'
          write(MsgOut,'(A)') 'md_exe_file           = ./namd2       '//&
               '# md exe file'
          write(MsgOut,'(A)') 'md_temperature        = 300.0         '//&
               '# md temperature'
          write(MsgOut,'(A)') 'md_steps              = 1000          '//&
               '# md steps / exchange'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'system_size_x         = 23.4          '//&
               '# system size --automatically create pme grids'
          write(MsgOut,'(A)') 'system_size_y         = 23.2          '//&
               '#'
          write(MsgOut,'(A)') 'system_size_z         = 23.9          '//&
               '#'
          write(MsgOut,'(A)') '# pme_grid              = yes         '//&
               '  # if you want to setup pme grids,(YES/[NO]) '
          write(MsgOut,'(A)') '# pme_grid_size_x       = 24          '//&
               '  # pme grid size x, y, z'
          write(MsgOut,'(A)') '# pme_grid_size_y       = 24          '//&
               '  #'
          write(MsgOut,'(A)') '# pme_grid_size_z       = 24          '//&
               '  #'
          write(MsgOut,'(A)') ' '
          write(MsgOut,'(A)') 'num_mpiproc_md        = 8             '//&
               '# core for MD'
          write(MsgOut,'(A)') ' '

        end if
      end if

      stop

    else
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*    INPUT_BUILDER: build input files for REIN     *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '

      ctrl_filename = arg1

    end if
    
    return
  
  end subroutine usage

end module ib_usage_mod
