!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------0---------0---------0---------0---------0---------0---------0---------0
!
!  Program: input_builder
!
!  Description: build REIN input files
!
!  Authors: Naoyuki Miyashita(NM)
!           Norio Takase(NT)
!
!  Date: 06 Jun, 2012
!
!--------0---------0---------0---------0---------0---------0---------0---------0

program input_builder

  use ib_build_mod
  use ib_setup_mod
  use ib_ctrl_mod
  use ib_usage_mod

  implicit none

  ! local variables
  character(100)     :: ctrl_filename
  type(s_ctrl_data)  :: ctrl_data
  type(s_build_info) :: build_info


  ! show usage
  !
  call usage(ctrl_filename)


  ! read control parameters for input builder
  !
  write(MsgOut,'(A)') '[STEP1] Read Control Parameters for input builder'
  write(MsgOut,'(A)') ' '

  call control(ctrl_filename, ctrl_data)


  ! setup build information
  !
  write(MsgOut,'(A)') '[STEP2] Setup build information'
  write(MsgOut,'(A)') ' '

  call setup(ctrl_data, build_info)


  ! build REIN input files
  !
  write(MsgOut,'(A)') '[STEP3] Build REIN input files'
  write(MsgOut,'(A)') ' '

  call build(ctrl_data, build_info)

      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*    INPUT_BUILDER: Bye                            *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '


  stop

end program input_builder
