/*
-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied
!    warranty of MERCHANTABILITY or FITNESS FOR
!    A PARTICULAR PURPOSE.
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General
!    Public License along with REIN.
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------
 */

//-----------------------------------------------------
#include "c_types.h"
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
//-----------------------------------------------------

#define MAX_CTRL_FILES    10
#define MAX_BUF          1024

//-----------------------------------------------------
typedef struct _String
{
  cChar*    cstr;
}
String;
//-----------------------------------------------------
typedef struct _Strings
{
  cInt      count;
  String*   strings;
}
Strings;
//-----------------------------------------------------
typedef struct _Inifile
{
  cChar     filename[256];
  Strings   lines;

} CtrlFile;
//-----------------------------------------------------

//-----------------------------------------------------
static CtrlFile* g_CtrlFiles[MAX_CTRL_FILES];
//-----------------------------------------------------
void copenctrlfile_(const cChar* filename, cInt64* handle)
{
  *handle = 0;

  cInt i, j;

  for(i = 0; i < MAX_CTRL_FILES; ++i)
    {
      if (g_CtrlFiles[i] != NULL &&
		  !strcmp(g_CtrlFiles[i]->filename, (cChar*)filename))
		{
		  *handle = (cInt64)g_CtrlFiles[i];
		  return ;
		}
    }

  for(i = 0; i < MAX_CTRL_FILES; ++i)
    {
      if (g_CtrlFiles[i] == NULL)
		break;
    }
  if (i == MAX_CTRL_FILES)
    return ;

  FILE* fp = fopen(filename, "r");
  if (fp == NULL)
    return ;

  g_CtrlFiles[i] = malloc(sizeof(CtrlFile));

  cInt CtrlFileIdx = i;
  CtrlFile* CtrlFile = g_CtrlFiles[CtrlFileIdx];

  memset(CtrlFile, 0, sizeof(CtrlFile));

  // Setup Filename
  strcpy(CtrlFile->filename, (cChar*)filename);

  // Read Data
  cChar buf[1024], buf2[MAX_BUF];

  cInt count = 0;
  while(fgets(buf, 1023, fp) != NULL)
    count ++;

  CtrlFile->lines.strings = malloc(sizeof(String) * count);
  for(i = 0; i < count; ++i)
    CtrlFile->lines.strings[i].cstr = NULL;

  fseek(fp, 0, SEEK_SET);

#define SET_STR                                       \
{                                                     \
  buf2[j++] = '\0';                                   \
  CtrlFile->lines.strings[count].cstr = malloc(j);     \
  strcpy(CtrlFile->lines.strings[count++].cstr, buf2); \
  add_spc = j = 0;                                    \
}

  cInt len, add_spc;

  count = j = add_spc = 0;

  while(fgets(buf, 1023, fp) != NULL)
    {
      len = strlen(buf);
      for(i = 0; i < len; ++i)
		{
		  if (buf[i] == '\n' || buf[i] == '\r' || buf[i] == '#')
			{
			  SET_STR;
			  break;
			}
		  else if (buf[i] == '\\')
			{
			  add_spc = 0;
			  break;
			}
		  else
			{
			  if (buf[i] != ' ' && buf[i] != '\t')
				{
				  add_spc = 1;
				  buf2[j++] = buf[i];
				}
			  else if (add_spc == 1)
				{
				  buf2[j++] = buf[i];
				}

			  if (j + 1 >= MAX_BUF)
				{
				  SET_STR;
				  break;
				}
			}
		}	
    }

  if (j != 0) 
    SET_STR;

  CtrlFile->lines.count = count;

  *handle = (cInt64)CtrlFile;

  fclose(fp);
}
//-----------------------------------------------------
void cclosectrlfile_(cInt64* handle)
{
  if (*handle == (cInt64)NULL)
    return ;

  cInt i, j;

  for(i = 0; i < MAX_CTRL_FILES; ++i)
    {
      if (*handle != (cInt64)g_CtrlFiles[i])
		continue;

      CtrlFile* CtrlFile = g_CtrlFiles[i];

      for(j = 0; j < CtrlFile->lines.count; ++j)
		free(CtrlFile->lines.strings[j].cstr);
      free(CtrlFile->lines.strings);
      free(CtrlFile);

      g_CtrlFiles[i] = NULL;
    }
}
//-----------------------------------------------------
void creadctrlfilestring_(cInt64* handle,
						  const cChar* section,
						  const cChar* key,
						  const cChar* def_value,
						  cChar* value,
						  cInt* value_len)
{
  if (*handle == 0)
    return ;

  cInt cvl = *value_len;

  memset(value, 0, cvl);
  cvl --;

  if (def_value != NULL && value != NULL)
    strncpy(value, (cChar *)def_value, cvl);

  if (section == NULL || key == NULL)
    return ;

  cInt i, j;

  for(i = 0; i < MAX_CTRL_FILES; ++i)
    if (*handle == (cInt64)g_CtrlFiles[i])
      break;
 
  if (i >= MAX_CTRL_FILES)
    return ;

  CtrlFile* CtrlFile = g_CtrlFiles[i];

  cInt len;
  cChar usection[256];
  cChar ukey[256];
  strcpy(usection, (cChar *)section);
  strcpy(ukey, (cChar *)key);
  len = strlen(usection);
  for(i = 0; i < len; ++i)
    usection[i] = toupper(usection[i]);
  len = strlen(ukey);
  for(i = 0; i < len; ++i)
    ukey[i] = toupper(ukey[i]);

  cChar str[MAX_BUF], *tok;

  if (strlen(usection) == 0)
    {
      i = 0;
      goto SEARCH_KEY;
    }

  for(i = 0; i < CtrlFile->lines.count; ++i)
    {
      strcpy(str, CtrlFile->lines.strings[i].cstr);
      tok = (cChar*)strtok(str, "[]");
      if (tok == NULL)
		continue ;

      while(tok != NULL)
		{
		  len = strlen(tok);
		  for(j = 0; j < len; ++j)
			tok[j] = toupper(tok[j]);

		  if (!strcmp(tok, usection))
			goto SEARCH_KEY;

		  tok = (cChar*)strtok(NULL, "[]");
		}
    }

  return ;

 SEARCH_KEY:

  i++;
  for( ; i < CtrlFile->lines.count; ++i)
    {
      strcpy(str, CtrlFile->lines.strings[i].cstr);

      if (strstr(str, "[") != NULL || strstr(str, "]") != NULL)
		break;

      // Read Key
      tok = (cChar*)strtok(str, "=");
      if (tok == NULL)
		continue ;

      while(*tok == ' ' || *tok == '\t')
		tok++;

      len = strlen(tok);
      for(j = 0; j < len; ++j)
		tok[j] = toupper(tok[j]);

      len --;
      while((*(tok+len) == ' ' || *(tok+len) == '\t') && len >= 0)
		{
		  *(tok+len) = '\0';
		  len--;
		}

      if (strcmp(tok, ukey))
		continue ;

      // Read Value
      tok = (cChar*)strtok(NULL, "=");
      if (tok == NULL)
		{
		  strcpy(value, "");
		  return ;
		}

      while(*tok == ' ' || *tok == '\t')
		tok++;

      len = strlen(tok);
      len --;
      while((*(tok+len) == ' ' || *(tok+len) == '\t') && len >= 0)
		{
		  *(tok+len) = '\0';
		  len--;
		}

      strncpy(value, tok, cvl);
      return ;
    }
}
//-----------------------------------------------------
void creadctrlfileinteger_(cInt64* handle,
						   const cChar* section,
						   const cChar* key,
						   cInt* def_value,
						   cInt* value)
{
  cChar value_str[256];

  cChar def_value_str[256];
  sprintf(def_value_str, "%d", *def_value);

  cInt value_len = 256;

  creadctrlfilestring_(handle, section, key, 
					   def_value_str, value_str, &value_len);

  if (strlen(value_str) != 0)
	*value = atoi(value_str);
}
//-----------------------------------------------------
void creadctrlfilereal_(cInt64* handle,
						const cChar* section,
						const cChar* key,
						cDouble* def_value,
						cDouble* value)
{
  cChar value_str[256];

  cChar def_value_str[256];
  sprintf(def_value_str, "%e", *def_value);

  cInt value_len = 256;

  creadctrlfilestring_(handle, section, key, 
					   def_value_str, value_str, &value_len);
  
  if (strlen(value_str) > 0)
	*value = atof(value_str);
}
//-----------------------------------------------------
