!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
! 
!  Module   messages
!> @brief   utilities for taking messages to Std or Err output
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM), Yuji Sugita (YS)
!! @date    2012/05/24
!! @note    MsgOut and ErrOut are defined here.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module messages_mod

  implicit none
  private

  ! parameters
  integer, public, parameter :: MsgOut  = 6
  integer, public, parameter :: ErrOut  = 6

  ! subroutines and functions
  public   :: error_msg
  public   :: error_msg_alloc
  public   :: error_msg_dealloc
  public   :: error_msg_fileio

contains
  
  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine error_msg
  !> @brief     write error message and stop execution
  !! @param[in] message Error message
  !! @authors   NT, YS
  !! @date      2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine error_msg(message)

    character(*), intent(in) :: message

    write(ErrOut,*) message
    write(ErrOut,*) ' '
    stop

  end subroutine error_msg

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine error_msg_alloc
  !> @brief     write error message in allocation and stop execution
  !! @param[in] message Error message
  !! @authors   NT, YS
  !! @date      2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine error_msg_alloc()

    call error_msg('Memory allocation error')

  end subroutine error_msg_alloc

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine error_msg_dealloc
  !> @brief     write error message in deallocation and stop execution
  !! @param[in] message Error message
  !! @authors   NT, YS
  !! @date      2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine error_msg_dealloc()

    call error_msg('Memory deallocation error')

  end subroutine error_msg_dealloc

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine error_msg_fileio
  !> @brief     write error message in fileio and stop execution
  !! @param[in] message Error message
  !! @authors   NT, YS
  !! @date      2010/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine error_msg_fileio()

    call error_msg('File I/O error')

  end subroutine error_msg_fileio

end module messages_mod
