!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   program_mod
!> @brief   utilities for program path
!! @authors Norio Takase(NT),Naoyuki Miyashita(NM), Yuji Sugita(YS)
!! @date    2012/06/04 
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module program_mod

  implicit none
  private

  ! subroutines and functions
  public   :: get_program_path
  public   :: get_program_name

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function    get_program_path
  !> @brief      get program path
  !! @authors    NT
  !! @return     character(200)  program path
  !! @date       2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  function get_program_path()

    character(200)            :: get_program_path

    ! local variables
    integer                   :: idx
    character(200)            :: path


    path = get_program_name()
    
    idx = SCAN(path, '/', .true.)
    if (idx == 0) then
      get_program_path = ''
      return
    end if

    get_program_path = path(:idx-1)
    return

  end function get_program_path


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function    get_program_name
  !> @brief      get program name
  !! @authors    NT
  !! @return     character(200)  program path
  !! @date       2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  function get_program_name()

    character(200)            :: get_program_name

    ! local variables
    character(200)            :: arg0, pwd
    character(1)              :: c


    call getarg(0, arg0)
    call getenv('PWD',pwd)

    c = arg0
    if (c /= '/') then
      if (c == '.' .or. SCAN(arg0, '/') /= 0) then
        get_program_name = TRIM(pwd)//'/'//arg0
      else
        get_program_name = ''
      end if
    else if (c == '/') then
      get_program_name = arg0
    end if

    return

  end function get_program_name

end module program_mod
