!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   replace_string_mod
!> @brief   utilities for replace string
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM), Yuji Sugita (YS)
!! @date    2012/05/31
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module replace_string_mod

  implicit none
  private

  ! subroutines and functions
  public  :: check_replace
  public  :: replace

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function    check_replace
  !> @brief      check replace keyword
  !! @authors    NT
  !! @param[in]  line      target string
  !! @param[out] kw_index  begin index of replace keyword
  !! @param[out] keyword   replace keyword
  !! @param[in]  delimiter keyword delimiter [optional] (def:"@")
  !! @return     logical   replace is possible or not
  !! @date       2012/05/31 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  function check_replace(line, kw_index, keyword, delimiter)

    logical           :: check_replace

    ! formal arguments
    character(*),     intent(in)           :: line
    integer,          intent(inout)        :: kw_index
    character(*),     intent(inout)        :: keyword
    character(*),     intent(in), optional :: delimiter

    ! local variables
    integer           :: bidx, eidx


    check_replace = .false.

    if (PRESENT(delimiter)) then
      bidx = SCAN(line, delimiter)
    else
      bidx = SCAN(line, '@')
    end if
    if (bidx == 0) &
      return
      
    if (PRESENT(delimiter)) then
      eidx = SCAN(line(bidx+1:), delimiter)
    else
      eidx = SCAN(line(bidx+1:), '@')
    end if
    if (eidx == 0) &
      return

    kw_index = bidx
    keyword = line(bidx+1:bidx+eidx-1)

    check_replace = .true.
    return

  end function check_replace


  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    replace
  !> @brief        replace keyword to string
  !! @authors      NT
  !! @param[inout] line     target string
  !! @param[in]    kw_idnex begin index of replace keyword
  !! @param[in]    keyword  replace keyword
  !! @param[in]    string   replace string
  !! @date         2012/05/31 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine replace(line, kw_index, keyword, string)

    ! formal arguments
    character(*),     intent(inout) :: line
    integer,          intent(in)    :: kw_index
    character(*),     intent(in)    :: keyword
    character(*),     intent(in)    :: string

    ! local variables
    character(1000)   :: work


    work = line(:kw_index-1) // &
           string(:LEN_TRIM(string)) // &
           line(kw_index+LEN_TRIM(keyword)+2:)
    line = work

    return

  end subroutine replace

end module replace_string_mod
