/*
-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied
!    warranty of MERCHANTABILITY or FITNESS FOR
!    A PARTICULAR PURPOSE.
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General
!    Public License along with REIN.
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------
 */

//------------------------------------------------------------------------------
#define _LARGEFILE64_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include "c_io_gromacs_cv1.h"
//------------------------------------------------------------------------------
static const cInt GRX_MAGIC = 1993;
//------------------------------------------------------------------------------
static cInt  g_grxCount = 0;
static grxInfo g_infos[MAX_GRX_FILE] = { 0 };
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
cInt byte_swap4i(cInt value, cInt doSwap)
{
  if (!doSwap)
    return value;

  cInt svalue;

  cChar* cvalue1 = (cChar *)&svalue;
  cChar* cvalue2 = (cChar *)&value;
  cvalue1[0] = cvalue2[3];
  cvalue1[1] = cvalue2[2];
  cvalue1[2] = cvalue2[1];
  cvalue1[3] = cvalue2[0];

  return svalue;
}
//------------------------------------------------------------------------------
cFloat byte_swap4r(cFloat value, cInt doSwap)
{
  if (!doSwap)
    return value;

  cFloat svalue;

  cChar* cvalue1 = (cChar *)&svalue;
  cChar* cvalue2 = (cChar *)&value;
  cvalue1[0] = cvalue2[3];
  cvalue1[1] = cvalue2[2];
  cvalue1[2] = cvalue2[1];
  cvalue1[3] = cvalue2[0];

  return svalue;
}
//------------------------------------------------------------------------------
cDouble byte_swap8r(cDouble value, cInt doSwap)
{
  if (!doSwap)
    return value;

  cDouble svalue;

  cChar* cvalue1 = (cChar *)&svalue;
  cChar* cvalue2 = (cChar *)&value;
  cvalue1[0] = cvalue2[7];
  cvalue1[1] = cvalue2[6];
  cvalue1[2] = cvalue2[5];
  cvalue1[3] = cvalue2[4];
  cvalue1[4] = cvalue2[3];
  cvalue1[5] = cvalue2[2];
  cvalue1[6] = cvalue2[1];
  cvalue1[7] = cvalue2[0];

  return svalue;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
cInt
calc_float_size(cInt box_size, 
		cInt x_size, 
		cInt v_size, 
		cInt f_size, 
		cInt natoms)
{
  static const cInt DIM = 3;

  cInt size = 0;

  if (box_size != 0)
    size = box_size / (DIM * DIM);
  else if (x_size != 0)
    size = x_size / (natoms * DIM);
  else if (v_size != 0)
    size = v_size / (natoms * DIM);
  else if (f_size != 0)
    size = f_size / (natoms * DIM);

  return size;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void
cinpgromacstrj_(cInt64*  handle, 
		cInt*    pnumAtoms,
		cDouble* coordinates,
		cDouble* pbcCoords,
		cInt*    ret)
{
  *ret = -1;

  grxInfo* info;
  cInt i;
  for(i = 0; i < MAX_GRX_FILE; ++i)
    if (*handle == (cInt64)&g_infos[i])
      break ;
  if (i == MAX_GRX_FILE)
    return ;
  info = &g_infos[i];

  FILE* fp = info->file;
  if (fp == NULL)
    {
      printf("ERROR cannot get FILE ptr\n");
      return;
    }

  cInt numAtoms = *pnumAtoms;

  cInt magic;
  if (fread((void*)&magic, sizeof(cInt), 1, fp) != 1)
    return ;
  cInt doSwap = (magic != GRX_MAGIC);
  magic = byte_swap4i(magic,doSwap);
  
  cInt iverlen1;
  if (fread((void*)&iverlen1, sizeof(cInt), 1, fp) != 1)
    return ;
  iverlen1 = byte_swap4i(iverlen1,doSwap);
  
  cInt iverlen2;
  if (fread((void*)&iverlen2, sizeof(cInt), 1, fp) != 1)
    return ;
  iverlen2 = byte_swap4i(iverlen2,doSwap);
  
  cChar version[256];
  memset(version,0,sizeof(cChar)*256);
  if (fread((void*)version, sizeof(cChar)*iverlen2, 1, fp) != 1)
    return ;
  
  cInt ir_size;
  if (fread((void*)&ir_size, sizeof(cInt), 1, fp) != 1)
    return ;
  ir_size = byte_swap4i(ir_size,doSwap);
  
  cInt e_size;
  if (fread((void*)&e_size, sizeof(cInt), 1, fp) != 1)
    return ;
  e_size = byte_swap4i(e_size,doSwap);
  
  cInt box_size;
  if (fread((void*)&box_size, sizeof(cInt), 1, fp) != 1)
    return ;
  box_size = byte_swap4i(box_size,doSwap);
  
  cInt vir_size;
  if (fread((void*)&vir_size, sizeof(cInt), 1, fp) != 1)
    return ;
  vir_size = byte_swap4i(vir_size,doSwap);
  
  cInt pres_size;
  if (fread((void*)&pres_size, sizeof(cInt), 1, fp) != 1)
    return ;
  pres_size = byte_swap4i(pres_size,doSwap);
  
  cInt top_size;
  if (fread((void*)&top_size, sizeof(cInt), 1, fp) != 1)
    return ;
  top_size = byte_swap4i(top_size,doSwap);
  
  cInt sym_size;
  if (fread((void*)&sym_size, sizeof(cInt), 1, fp) != 1)
    return ;
  sym_size = byte_swap4i(sym_size,doSwap);
  
  cInt x_size;
  if (fread((void*)&x_size, sizeof(cInt), 1, fp) != 1)
    return ;
  x_size = byte_swap4i(x_size,doSwap);
  
  cInt v_size;
  if (fread((void*)&v_size, sizeof(cInt), 1, fp) != 1)
    return ;
  v_size = byte_swap4i(v_size,doSwap);
  
  cInt f_size;
  if (fread((void*)&f_size, sizeof(cInt), 1, fp) != 1)
    return ;
  f_size = byte_swap4i(f_size,doSwap);
  
  cInt natoms;
  if (fread((void*)&natoms, sizeof(cInt), 1, fp) != 1)
    return ;
  natoms = byte_swap4i(natoms,doSwap);
  
  cInt step;
  if (fread((void*)&step, sizeof(cInt), 1, fp) != 1)
    return ;
  step = byte_swap4i(step,doSwap);
  
  cInt nre;
  if (fread((void*)&nre, sizeof(cInt), 1, fp) != 1)
    return ;
  nre = byte_swap4i(nre,doSwap);
  
  if (natoms != numAtoms)
    {
      printf(" inp_gromacs> ERROR: different atom count from REF coords.\n");
      return ;
    }

  cInt floatsize = calc_float_size(box_size,x_size,v_size,f_size,natoms);
  
  if (floatsize != 8 && floatsize != 4)
    return ;
  
  cDouble t, lambda;
  if (floatsize == 4)
    {
      cFloat t4, lambda4;
      if (fread((void*)&t4, sizeof(cFloat), 1, fp) != 1)
	return ;
      if (fread((void*)&lambda4, sizeof(cFloat), 1, fp) != 1)
	return ;
      t = byte_swap4r(t4,doSwap);
      lambda = byte_swap4r(lambda4,doSwap);
    }
  else
    {
      if (fread((void*)&t, sizeof(cDouble), 1, fp) != 1)
	return ;
      if (fread((void*)&lambda, sizeof(cDouble), 1, fp) != 1)
	return ;
      t = byte_swap8r(t,doSwap);
      lambda = byte_swap8r(lambda,doSwap);
    }
  
  if (info->firstIO)
    {
      info->firstIO = 0;

      printf(" inp_gromacs_trj> magic = %d\n", magic);
      printf(" inp_gromacs_trj> verlen1 = %d\n", iverlen1);
      printf(" inp_gromacs_trj> verlen2 = %d\n", iverlen2);
      printf(" inp_gromacs_trj> version = %s\n", version);
      printf(" inp_gromacs_trj> ir_size = %d\n", ir_size);
      printf(" inp_gromacs_trj> e_size = %d\n", e_size);
      printf(" inp_gromacs_trj> box_size = %d\n", box_size);
      printf(" inp_gromacs_trj> vir_size = %d\n", vir_size);
      printf(" inp_gromacs_trj> pres_size = %d\n", pres_size);
      printf(" inp_gromacs_trj> top_size = %d\n", top_size);
      printf(" inp_gromacs_trj> sym_size = %d\n", sym_size);
      printf(" inp_gromacs_trj> x_size = %d\n", x_size);
      printf(" inp_gromacs_trj> v_size = %d\n", v_size);
      printf(" inp_gromacs_trj> f_size = %d\n", f_size);
      printf(" inp_gromacs_trj> natoms = %d\n", natoms);
      printf(" inp_gromacs_trj> step = %d\n", step);
      printf(" inp_gromacs_trj> nre = %d\n", nre);
      printf(" inp_gromacs_trj> floatsize = %d\n", floatsize);
    }
      
  if (box_size != 0)
    {
      if (floatsize == 4)
	{
	  cFloat v;
	  cInt total, j, k;
	      
	  total = 0;
	  for(j = 0; j < 3; ++j)
	    for(k = 0; k < 3; ++k)
	      {
		if (fread((void*)&v, sizeof(cFloat), 1, fp) != 1)
		  return ;
		pbcCoords[total++] = 10 * byte_swap4r(v,doSwap);
	      }
	}
      else if (doSwap)
	{
	  cDouble v;
	  cInt total, j, k;
	  
	  total = 0;
	  for(j = 0; j < 3; ++j)
	    for(k = 0; k < 3; ++k)
	      {
		if (fread((void*)&v, sizeof(cDouble), 1, fp) != 1)
		  return ;
		pbcCoords[total++] = 10 * byte_swap8r(v,1);
	      }
	}
      else
	{
	  if (fread((void*)&pbcCoords, sizeof(cChar), box_size, fp) != box_size)
	    return ;
	  cInt j;
	  for(j = 0; j < box_size / floatsize; ++j)
	    pbcCoords[j] *= 10.0;
	}
    }
  
  fseek(fp, vir_size, SEEK_CUR);
  fseek(fp, pres_size, SEEK_CUR);
  
  if (x_size != 0)
    {
      if (floatsize == 4)
	{
	  cFloat v;
	  cInt total, j, k;
	  
	  total = 0;
	  for(j = 0; j < natoms; ++j)
	    for(k = 0; k < 3; ++k)
	      {
		if (fread((void*)&v, sizeof(cFloat), 1, fp) != 1)
		  return ;
		coordinates[total++] = 10 * byte_swap4r(v,doSwap);
	      }
	}
      else if (doSwap)
	{
	  cDouble v;
	  cInt total, j, k;
	  
	  total = 0;
	  for(j = 0; j < natoms; ++j)
	    for(k = 0; k < 3; ++k)
	      {
		if (fread((void*)&v, sizeof(cDouble), 1, fp) != 1)
		  return ;
		coordinates[total++] = 10 * byte_swap8r(v,1);
	      }
	    }
      else
	{
	  if (fread((void*)&coordinates, sizeof(cChar), x_size, fp) != x_size)
	    return ;
	  cInt j;
	  for(j = 0; j < x_size / floatsize; ++j)
	    coordinates[j] *= 10.0;
	}
    }
  
  fseek(fp, v_size, SEEK_CUR);
  fseek(fp, f_size, SEEK_CUR);

  *ret = 0;
}
//------------------------------------------------------------------------------
void
coutgromacstrjsel_(cInt64*  handle, 
		   cInt*    pnumAtomIndices,
		   cInt*    atomIndices,
		   cDouble* coordinates,
		   cDouble* pbcCoords,
		   cInt*    poutPBCCoords,
		   cInt*    ret)
{
  *ret = -1;

  grxInfo* info;
  cInt i;
  for(i = 0; i < MAX_GRX_FILE; ++i)
    if (*handle == (cInt64)&g_infos[i])
      break ;
  if (i == MAX_GRX_FILE)
    return ;
  info = &g_infos[i];

  FILE* fp = info->file;
  if (fp == NULL)
    {
      printf("ERROR cannot get FILE ptr\n");
      return;
    }

  const cInt FLOAT_SIZE = 4;

  cInt numAtomIndices = *pnumAtomIndices;
  cInt outPBCCoords = *poutPBCCoords;

  if (numAtomIndices == 0)
    return ;

  if (fwrite((void*)&GRX_MAGIC, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt iverlen1 = 13;
  if (fwrite((void*)&iverlen1, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt iverlen2 = 12;
  if (fwrite((void*)&iverlen2, sizeof(cInt), 1, fp) != 1)
    return ;

  cChar version[256];
  memset(version, 0, sizeof(cChar)*256);
  sprintf(version,"GMX_trn_file");
  if (fwrite((void*)version,sizeof(cChar)*iverlen2, 1, fp) != 1)
    return ;

  cInt ir_size = 0;
  if (fwrite((void*)&ir_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt e_size = 0;
  if (fwrite((void*)&e_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt box_size = outPBCCoords ? 9 * FLOAT_SIZE : 0;
  if (fwrite((void*)&box_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt vir_size = 0;
  if (fwrite((void*)&vir_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt pres_size = 0;
  if (fwrite((void*)&pres_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt top_size = 0;
  if (fwrite((void*)&top_size, sizeof(cInt), 1, fp) != 1)
    return ;
  
  cInt sym_size = 0;
  if (fwrite((void*)&sym_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt x_size = numAtomIndices * FLOAT_SIZE * 3;
  if (fwrite((void*)&x_size, sizeof(cInt), 1, fp) != 1)
    return ;
  
  cInt v_size = 0;
  if (fwrite((void*)&v_size, sizeof(cInt), 1, fp) != 1)
    return ;
  
  cInt f_size = 0;
  if (fwrite((void*)&f_size, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt natoms = numAtomIndices;
  if (fwrite((void*)&natoms, sizeof(cInt), 1, fp) != 1)
    return ;
  
  cInt step = info->step++;
  if (fwrite((void*)&step, sizeof(cInt), 1, fp) != 1)
    return ;

  cInt nre = 0;
  if (fwrite((void*)&nre, sizeof(cInt), 1, fp) != 1)
    return ;

  cFloat t = 0.0f;
  if (fwrite((void*)&t, sizeof(cFloat), 1, fp) != 1)
    return ;

  cFloat lambda = 0.0f;
  if (fwrite((void*)&lambda, sizeof(cFloat), 1, fp) != 1)
    return ;

  if (box_size != 0)
    {
      cFloat v;
      cInt total, j, k;

      total = 0;
      for(j = 0; j < 3; ++j)
	for(k = 0; k < 3; ++k)
	  {
	    v = (cFloat)(pbcCoords[total++] / 10.0);
	    if (fwrite((void*)&v, sizeof(cFloat), 1, fp) != 1)
	      return ;
	  }
    }

  if (x_size != 0)
    {
      cFloat v;
      cInt index, j, k;

      for(j = 0; j < natoms; ++j)
	{
	  index = atomIndices[j] - 1;
	  for(k = 0; k < 3; ++k)
	    {
	      v = (cFloat)(coordinates[index * 3 + k] / 10.0);
	      if (fwrite((void*)&v, sizeof(cFloat), 1, fp) != 1)
		return ;
	    }
	}
    }

  *ret = 0;

}
//------------------------------------------------------------------------------
void
copengromacstrj_(cInt64* handle, cInt* pisInput, const cChar* filename)
{
  cInt isInput = *pisInput;

  *handle = 0;

  if (filename == NULL)
    return ;

  if (g_grxCount >= MAX_GRX_FILE)
    return ;

  FILE* fp = (FILE *)GRX_FOPEN(filename,isInput >= 2 ? "rb" : "wb");
  if (fp == NULL)
    return ;

  // Initialize variables
  cInt i;
  for(i = 0; i < MAX_GRX_FILE; ++i)
    if (g_infos[i].file == 0)
      {
	memset(&g_infos[i], 0, sizeof(grxInfo));

	g_infos[i].file = fp;
	g_infos[i].firstIO = 1;
	g_infos[i].step = 0;
	*handle = (cInt64)&g_infos[i];

	g_grxCount ++;

	break;
      }
}
//------------------------------------------------------------------------------
void
cclosegromacstrj_(cInt64* handle)
{
  grxInfo* info;
  cInt i;
  for(i = 0; i < MAX_GRX_FILE; ++i)
    if (*handle == (cInt64)&g_infos[i])
      break ;
  if (i == MAX_GRX_FILE)
    return ;
  info = &g_infos[i];

  // close file
  fclose(info->file);

  info->file = 0;
  g_grxCount --;
}
//------------------------------------------------------------------------------
