!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------0---------0---------0---------0---------0---------0---------0---------0
!
!  Program: remd_convert
!
!  Description: convert REMD trajectory format
!
!  Authors: Norio Takase (NT), Naoyuki Miyashita(NM), Yuji Sugita (YS)
!
!  Date: 24 May, 2012
!
!--------0---------0---------0---------0---------0---------0---------0---------0

program remd_convert

  use remdcnv_run_mod
  use remdcnv_setup_mod
  use remdcnv_ctrl_mod
  use remdcnv_usage_mod
  use cnv_data_str_mod
  use molecules_str_mod
  use selatoms_str_mod

  implicit none

  ! local variables
  logical            :: use_pdb
  integer            :: i
  character(100)     :: ctrl_filename
  type(s_ctrl_data)  :: ctrl_data

  type(s_cnv_data)   :: cnv_data
  type(s_molecule)   :: molecule
  type(s_selatoms)   :: trjout_atom


  ! show usage
  !
  call usage(ctrl_filename)


  ! read control parameters for conversion
  !
  write(MsgOut,*) '[STEP1] Read Control Parameters for conversion'
  write(MsgOut,*) ' '

  call control(ctrl_filename, ctrl_data)


  ! set relevant variables and structures
  !
  write(MsgOut,*) '[STEP2] Set relevant variables and structures'
  write(MsgOut,*) ' '

  call setup(ctrl_data, cnv_data, molecule, trjout_atom)


  ! start convert
  !

  write(MsgOut,*) '[STEP3] Start convert'
  write(MsgOut,*) ' '

  call run(cnv_data, molecule, trjout_atom)


  ! deallocate memory
  !
  
  write(MsgOut,*) '[STEP4] Deallocate memory'
  write(MsgOut,*) ' '

  call dealloc_molecules_all(molecule)
  call dealloc_selatoms(trjout_atom)

      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*    REMD_CONVETER: Bye                            *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '


  stop

end program remd_convert
