!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   remdcnv_usage_mod
!> @brief   show usage of REMD convert
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM)
!! @date    2012/05/24 
! 
!--------1---------2---------3---------4---------5---------6---------7---------8

module remdcnv_usage_mod

  use messages_mod

  implicit none

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  usage
  !> @brief      show usage of remd_convert
  !! @authors    NT, NM
  !! @param[in]  arg1    control file name
  !! @date       2012/05/24
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine usage(arg1)

    ! formal arguments
    character(*), intent(out) :: arg1

    ! local variables
    character(80)             :: arg2


    call getarg(1, arg1)

    if (iargc() < 1      .or. &
         arg1 == '-help' .or. &
         arg1 == '-h'    .or. &
         arg1 == '-HELP' .or. &
         arg1 == '-H') then

      call getarg(2, arg2)

      if (iargc() < 2 .or. arg2 /= 'ctrl') then
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# normal usage'
        write(MsgOut,'(A)') '  % ./remd_convert INPCNV'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# usage to see control parameters'
        write(MsgOut,'(A)') '  % ./remd_convert -h ctrl'
        write(MsgOut,'(A)') ' '
      else
        write(MsgOut,'(A)') '# control parameters in remd_convert'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '[INPUT]'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'psffile           = initial.psf'
        write(MsgOut,'(A)') 'reffile           = initial.pdb'
        write(MsgOut,'(A)') 'trj_format        = DCD          '//&
             '# (DCD/MARBLE/PDB)'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'directory001      = ./results/result0000001_0000010'
        write(MsgOut,'(A)') 'directory002      = ./results/result0000011_0000020'
        write(MsgOut,'(A)') 'directory003      = ./results/result0000021_0000030'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'repeat001         = 3'
        write(MsgOut,'(A)') 'trj_filename001   = rep.{rep_no}/'//&
             'rep.{rep_no}.{step}.dcd'
        write(MsgOut,'(A)') 'energy_trj001     = replica.trj'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'energy_column     =              '//&
             '# column of output energy, ex. = 1 2-4, empty: all column'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '[OUTPUT]'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'out_pdbfile       = out.pdb'
        write(MsgOut,'(A)') 'trj_filename      = out.{type}.{nu'//&
             'mber}.{extension}'
        write(MsgOut,'(A)') 'trj_format        = DCD          #'//&
             ' (DCD/MARBLE/AMBER/TINKER/PDB/GROMACS)'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '[CONVERT]'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') 'type              = REPLICA      #'//&
             ' (REPLICA/CONDITION)'
        write(MsgOut,'(A)') 'rank              =              #'//&
             '  ex. = 1 2-8,  empty: output all rank '
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '[SELECTION]'
        write(MsgOut,'(A)') ' '
        write(MsgOut,'(A)') '# select_atom       = all'
        write(MsgOut,'(A)') '# select_atom       = molname:protein | molname:lipid'
        write(MsgOut,'(A)') '# mole_name001      = protein  P1:1:TYR P1:5:MET'
        write(MsgOut,'(A)') '# mole_name002      = lipid    OLEO:PCGL:OLEO'
        write(MsgOut,'(A)') ' '
      endif

      stop

    else
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'*   REMD_CONVERT: convert REMD trajectory format   *'
      write(MsgOut,'(A)')'*                                                  *'
      write(MsgOut,'(A)')'****************************************************'
      write(MsgOut,'(A)')' '
    end if
    
    return
  
  end subroutine usage

end module remdcnv_usage_mod
