!-------------------------------------------------------------------------------
!    This source code is part of
!    REIN (Replica-Exchange INterface program) version 1.0
!
!    Written by Naoyuki Miyashita and Yuji Sugita
!    Copyright (c) 2012, RIKEN, Japan
!
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License
!    as published by the Free Software Foundation, either 
!    version 3 of the License, or (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied 
!    warranty of MERCHANTABILITY or FITNESS FOR 
!    A PARTICULAR PURPOSE.  
!    See the GNU General Public License for more details.
!
!    You should have received a copy of the GNU General 
!    Public License along with REIN.  
!    If not, see <http://www.gnu.org/licenses/>.
!-------------------------------------------------------------------------------

!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   selatoms_str_mod
!> @brief   structure for atom selection
!! @authors Norio Takase (NT), Naoyuki Miyashita (NM), Yuji Sugita (YS)
!! @date    2012/05/24 
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module selatoms_str_mod

  use messages_mod

  implicit none
  private

  type, public :: s_selatoms
    integer, pointer :: idx(:) => null()
  end type s_selatoms

  public :: alloc_selatoms
  public :: dealloc_selatoms

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    alloc_selatoms
  !> @brief        allocate selatoms
  !! @authors      NT, YS
  !! @param[inout] selatoms   : list of atom index
  !! @param[in]    list_size  : size of list
  !! @date         2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine alloc_selatoms(selatoms, list_size)

    ! formal arguments
    type(s_selatoms),     intent(inout) :: selatoms
    integer,              intent(in)    :: list_size

    ! local variables
    integer                              :: alloc_stat

    call dealloc_selatoms(selatoms)

    allocate(selatoms%idx(list_size), stat = alloc_stat)
    if (alloc_stat /= 0) call error_msg_alloc

    selatoms%idx(1:list_size) = 0

    return

  end subroutine alloc_selatoms

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    dealloc_selatoms
  !> @brief        deallocate selatoms
  !! @authors      NT, YS
  !! @param[inout] selatoms   : list of selected atom index
  !! @param[in]    list_size  : size of list
  !! @date         2012/05/24 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine dealloc_selatoms(selatoms)

    ! formal arguments
    type(s_selatoms),     intent(inout) :: selatoms

    ! local variables
    integer                             :: dealloc_stat

    dealloc_stat = 0

    if (associated(selatoms%idx)) then
      deallocate(selatoms%idx, stat = dealloc_stat)
      selatoms%idx => null()
    end if

    if (dealloc_stat /= 0) call error_msg_dealloc

    return

  end subroutine dealloc_selatoms

end module selatoms_str_mod
